/*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "errno.h"
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include "sys/wait.h"

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

void
xcave_fill_filtered_model (gpointer		data,
							gint			colonne)
{
	GtkWidget			*TreeList;
	GtkWidget			*EntryColor;
	GtkListStore		*TreeList_filtered_model;
    GList 				*tmp_list_stock;
	GtkWidget			*treeview;
	GtkTreeModel 		*model;
	GtkTreeSelection	*selection;
	GtkTreeIter			iter;
	GtkTreeIter			child;
	GtkTreeIter			grandson;
	GtkTreeIter			parent;
	GtkTreeIter			grandpa;

    ptrVin 	    		tmpVin;
const	gchar				*pcolor;
	gchar				*pAppellation;
	gchar				*pRegion;
	gchar				*pcountry;
	gint				tmpcol;
	gint 				nbrow;

    XCAVE_DEBUG_B;

    tmpVin = g_malloc (sizeof (Vin));

	/*
	 * cleanup the treelist 
	 */
	TreeList = lookup_widget(window_xcave,"TreeList");
	TreeList_filtered_model	=(GtkListStore *)  gtk_tree_view_get_model((GtkTreeView *)TreeList); /* C */
	gtk_list_store_clear(GTK_LIST_STORE(TreeList_filtered_model)); /* C */

	/*
	 * get the selected entry_color 
	 * needed to display only wine corresponding to the selected color
	 */
	EntryColor = lookup_widget(window_xcave,"entry_color");
	pcolor =(const gchar*) gtk_entry_get_text    ((GtkEntry *)EntryColor);

	/*
	 * get the selected row of the treeview 
	 * needed to display only wine belonging to country/region/appellation 
	 * selected
	 * if no row is selected then (tmpcol = -1) to display all wine on the 
	 * stock
	 */
	treeview = lookup_widget(window_xcave,"TreeView");
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));  
	gtk_tree_selection_get_selected (selection, &model, &iter);
	pAppellation = g_malloc(appellation_size);
	nbrow = gtk_tree_selection_count_selected_rows(selection);
	
	if (nbrow == 0)
		tmpcol = -1;
	else {
		gtk_tree_model_get (model, &iter, COL_TREEVIEW, &pAppellation, -1);
		/*
		 * Determine level the selected data is 
		 */
		if (gtk_tree_model_iter_children(model, &child, &iter) == FALSE) {
			tmpcol = COL_TREELIST_APPELLATION;
		
			pRegion	= g_malloc(region_size);
			pcountry	= g_malloc(country_size);
		
			gtk_tree_model_iter_parent(model, &parent, &iter);
			gtk_tree_model_get (model, &parent, COL_TREEVIEW, &pRegion, -1);
			gtk_tree_model_iter_parent(model, &grandpa, &parent);
			gtk_tree_model_get (model, &grandpa, COL_TREEVIEW, &pcountry, -1);
		
		}
		else 
			if (gtk_tree_model_iter_children(model, &grandson, &child) == FALSE) 
				tmpcol = COL_TREELIST_REGION;
			else 
				tmpcol = COL_TREELIST_COUNTRY;
	}
		
	/*
	 * ok now it's time to play the game. I mean to glance through the stocks
	 * to display wine depending of different selection
	 */
	for(tmp_list_stock = list_stock; tmp_list_stock != NULL; tmp_list_stock = tmp_list_stock->next) {
        GtkTreeIter pIter;

		memset(tmpVin,'\0',sizeof(Vin));
		memcpy(tmpVin,tmp_list_stock->data,sizeof(Vin));

		if ( /* entry_color */
 			 ( 
			   (strcasecmp(pcolor,_("ALL")) == 0)
			   ||
			   (strcasecmp(pcolor,_("All")) == 0)
			   ||
		       (strcasecmp(pcolor,tmpVin->color) == 0)
			  )
			&&	/* treelist selection when entry_color is changed */	
			 ( (tmpcol == COL_TREELIST_APPELLATION && strcasecmp(pAppellation,tmpVin->Appellation) == 0)
			   ||
			   (tmpcol == COL_TREELIST_REGION && strcasecmp(pAppellation,tmpVin->Region) == 0)
			   ||
			   (tmpcol == COL_TREELIST_COUNTRY && strcasecmp(pAppellation,tmpVin->country) == 0)
			   ||
			   (tmpcol == -1)
			 )
			&&	/* treelist selection when treelist is changed */	
			 ( (colonne == COL_TREELIST_APPELLATION && 
				strcasecmp(data,tmpVin->Appellation) == 0 &&
				strcasecmp(pRegion,tmpVin->Region)   == 0 &&
				strcasecmp(pcountry,tmpVin->country)       == 0
			    )
			   ||
			   (colonne == COL_TREELIST_REGION && strcasecmp(data,tmpVin->Region) == 0)
			   ||
			   (colonne == COL_TREELIST_COUNTRY && strcasecmp(data,tmpVin->country) == 0)
			   ||
			   (colonne == COL_TREELIST_COLOR && strcasecmp(data,tmpVin->color) == 0)
			   ||
			   (colonne == COL_TREELIST_COLOR && strcasecmp(data,_("All")) == 0)
			   ||
			   (colonne == COL_TREELIST_COLOR && strcasecmp(data,_("ALL")) == 0)
			 )
		   && /* quantity to display drinking wine that are not deleted from
			   * the stock
			   */
		     ( (QT == 0 && tmpVin->qty != 0 )  || ( QT == 1 ) )
			) {
				gtk_list_store_append(TreeList_filtered_model, &pIter);/* C */
				gtk_list_store_set(TreeList_filtered_model, &pIter, COL_TREELIST_MILLESIME, (guint)tmpVin->vintage,
									COL_TREELIST_QTY, (guint)tmpVin->qty,
									COL_TREELIST_DOMAIN, tmpVin->Nom,
									COL_TREELIST_COUNTRY, tmpVin->country,
									COL_TREELIST_REGION, tmpVin->Region,
									COL_TREELIST_APPELLATION, tmpVin->Appellation,
									COL_TREELIST_COLOR, tmpVin->color,
									COL_TREELIST_MATURITE, xcave_compute_maturity_value(
												tmpVin->qty,tmpVin->vintage,tmpVin->Conserv),
									COL_TREELIST_PDATA, tmp_list_stock->data,
									COL_TREELIST_PGLIST, tmp_list_stock,
									-1);			
			}
	
	}

	gtk_tree_sortable_set_sort_column_id (	GTK_TREE_SORTABLE (TreeList_filtered_model),
											COL_TREELIST_MATURITE, 
											GTK_SORT_DESCENDING);

	gtk_tree_view_set_model(GTK_TREE_VIEW(TreeList), GTK_TREE_MODEL(TreeList_filtered_model));

	g_free(tmpVin);
	g_free(pAppellation);

    XCAVE_DEBUG_E;
}

void
xcave_fill_model (GtkTreeStore * TreeView, GtkListStore * TreeList)
{
    GList 			*tmp_list_stock;
    ptrVin     		tmpVin;
	gchar			*pCountry;
	gchar			*pRegCountry;
	gchar			*pAppCountry;
	gchar			*pRegion;
	gchar			*pAppRegion;
	gchar			*pAppellation;

    XCAVE_DEBUG_B;

    tmpVin = g_malloc (sizeof (Vin));

	pCountry		= g_malloc(country_size);
	pRegCountry		= g_malloc(country_size);
	pAppCountry		= g_malloc(country_size);
	pRegion			= g_malloc(region_size);
	pAppRegion			= g_malloc(region_size);
	pAppellation	= g_malloc(appellation_size);

	strcpy (pCountry, ".");
    strcpy (pRegion, ".");
    strcpy (pAppellation, ".");
	strcpy (pRegCountry,".");
	strcpy (pAppCountry,".");
	strcpy (pAppRegion, ".");

	for(tmp_list_stock = list_stock; tmp_list_stock != NULL; tmp_list_stock = tmp_list_stock->next) {
        GtkTreeIter pIter;
        GtkTreeIter pIter2;
        GtkTreeIter pIter3;
        GtkTreeIter pIter4;

		memset(tmpVin,'\0',sizeof(Vin));
		memcpy(tmpVin,tmp_list_stock->data,sizeof(Vin));

		/*
		 * Affichage des vins consomms - dont la quantit est nulle dans le stocks
	     * mais dont la trace a neanmoins ete conservee
		*/
		if ( (QT == 0 && tmpVin->qty != 0 )  || ( QT == 1 ) ){
			if (  strcasecmp (tmpVin->country,pCountry)        != 0 ) {
				/* new country */
				gtk_tree_store_append(TreeView, &pIter, NULL);/* C */
				strncpy (pCountry,tmpVin->country,country_size);
				gtk_tree_store_set(TreeView, &pIter, COL_TREEVIEW, pCountry, -1);			
			}
			if (  strcasecmp (tmpVin->Region,pRegion)       != 0 ||
				  (  strcasecmp (tmpVin->Region,pRegion)       == 0 &&
				     strcasecmp (tmpVin->country,pRegCountry)        != 0    )) {
				/* new region */
				gtk_tree_store_append(TreeView, &pIter2, &pIter);/* C */
				strncpy (pRegion, tmpVin->Region, region_size);
				gtk_tree_store_set(TreeView, &pIter2, COL_TREEVIEW, pRegion, -1);			
			}
		
			if (  (  strcasecmp (tmpVin->Appellation,pAppellation)     != 0)  
				  ||
				  (  strcasecmp (tmpVin->Appellation,pAppellation)     == 0 &&
				     strcasecmp (tmpVin->Region,pAppRegion)            != 0 ) 
			      ||
			      (  strcasecmp (tmpVin->Appellation,pAppellation)     == 0 &&
				     strcasecmp (tmpVin->Region,pAppRegion)            == 0 && 
					 strcasecmp (tmpVin->country,pAppCountry)             != 0)
			   ) {
				/* new appellation */ 
				gtk_tree_store_append(TreeView, &pIter3, &pIter2);/* C */
				strncpy (pAppellation, tmpVin->Appellation,  appellation_size);
				gtk_tree_store_set(TreeView, &pIter3, COL_TREEVIEW, pAppellation, -1);			
			}

			gtk_list_store_append(TreeList, &pIter4);/* C */
			gtk_list_store_set(TreeList, &pIter4, 
						COL_TREELIST_MILLESIME, (guint)tmpVin->vintage,
						COL_TREELIST_QTY, (guint)tmpVin->qty,
						COL_TREELIST_DOMAIN, tmpVin->Nom,
						COL_TREELIST_COUNTRY, tmpVin->country,
						COL_TREELIST_REGION, tmpVin->Region,
						COL_TREELIST_APPELLATION, tmpVin->Appellation,
						COL_TREELIST_COLOR, tmpVin->color,
						COL_TREELIST_MATURITE, xcave_compute_maturity_value(
								tmpVin->qty,tmpVin->vintage,tmpVin->Conserv),
						COL_TREELIST_PDATA, tmp_list_stock->data,
						COL_TREELIST_PGLIST, tmp_list_stock,
						-1);			
			strncpy (pRegCountry,tmpVin->country,country_size);
			strncpy (pAppCountry,tmpVin->country,country_size);
			strncpy (pAppRegion, tmpVin->Region, region_size);
		}
	
	}

	gtk_tree_sortable_set_sort_column_id (	GTK_TREE_SORTABLE (TreeList),
											COL_TREELIST_MATURITE, 
											GTK_SORT_DESCENDING);
	g_free(tmpVin);
    g_free(pCountry); 
    g_free(pRegCountry); 
    g_free(pAppCountry); 
    g_free(pAppRegion);
    g_free(pRegion);
    g_free(pAppellation);
    XCAVE_DEBUG_E;
}

void
on_xcave_realize (	GtkWidget * widget,
					gpointer user_data)
{
	GtkWidget 			*st;
	GtkTreeStore		*StoreView;
	GtkListStore		*StoreList;
	
	GtkWidget			*scrolledwindow1;
	GtkWidget			*TreeView;
	GtkWidget			*scrolledwindow2;
	GtkWidget			*TreeList;
	GtkWidget			*hpaned1;

	GtkTreeViewColumn   *col;
	GtkCellRenderer     *renderer;

    XCAVE_DEBUG_B;

  window_xcave = widget;		/* assign main window in global for sub procedure treatments */

  st = lookup_widget (widget, "statusvins");
  if (st == NULL) g_warning ("Widget not found: statusvins");

  ctxtid_stock = gtk_statusbar_get_context_id ((GtkStatusbar *)st, "default");
  statid_stock = 0;


  /*
   * initialisation diverses et variees:
   *   (GList) liste du stock
   *   (GList) liste des country
   *   (GList) liste des regions
   *   (GList) liste des appellations
   *   (GList) liste des cepages
   */
	list_append_stock();
	
	list_append_rack();
	
	list_append_categ();
	list_append_country();
	list_append_region();
	list_append_appellation();
	list_append_cepage();
	
	list_country_append_stock();
	list_region_append_stock();
	list_appellation_append_stock();
	list_cepage_append_stock();

	list_country = g_list_sort(list_country,(GCompareFunc)cmp_country);
    list_region = g_list_sort(list_region,(GCompareFunc)cmp_region);
	list_appellation = g_list_sort(list_appellation,(GCompareFunc)cmp_appellation);
	list_cepage = g_list_sort(list_cepage,(GCompareFunc)cmp_cepage);

	/*
	* TreeView & TreeList init
	*/
	hpaned1 = lookup_widget(window_xcave,"hpaned1");

    /* Creates a new tree store as with n_columns columns each of the types passed in. */
    StoreView = gtk_tree_store_new(NUM_COLS_TREEVIEW, G_TYPE_STRING);

	/* Creates a new tree store as with n_columns columns each of the types passed in. */
	StoreList = gtk_list_store_new(NUM_COLS_TREELIST, 
				G_TYPE_UINT, 
				G_TYPE_UINT, 
				G_TYPE_STRING, 
				G_TYPE_STRING, 
				G_TYPE_STRING, 
				G_TYPE_STRING, 
				G_TYPE_STRING, 
				G_TYPE_INT,
				G_TYPE_POINTER,
				G_TYPE_POINTER);
													
	/* create scrollbar */
	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_paned_pack1 (GTK_PANED (hpaned1), scrolledwindow1, TRUE, TRUE);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	
	/* create view TreeView */
	TreeView = gtk_tree_view_new_with_model (GTK_TREE_MODEL(StoreView));
	gtk_widget_show (TreeView);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), TreeView);
	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (TreeView), FALSE);
	
	/* create scrollbar */
	scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow2);
	gtk_paned_pack2 (GTK_PANED (hpaned1), scrolledwindow2, TRUE, TRUE);
	GTK_WIDGET_UNSET_FLAGS (scrolledwindow2, GTK_CAN_FOCUS);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	
	/* create view TreeList */
	TreeList = gtk_tree_view_new_with_model (GTK_TREE_MODEL(StoreList));
	gtk_widget_show (TreeList);
	gtk_container_add (GTK_CONTAINER (scrolledwindow2), TreeList);
	
	xcave_fill_model (StoreView, StoreList);

	/* --- TreeView Column #1 --- */
        renderer = gtk_cell_renderer_text_new();
        col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREEVIEW,NULL);
        gtk_tree_view_append_column(GTK_TREE_VIEW(TreeView), col);
	
	/* --- TreeList Column #1 --- */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, _("Vintage"));
	gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList), col);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func(col, renderer, maturite_cell_data_function, NULL, NULL);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_TREELIST_MILLESIME);
	gtk_tree_view_column_set_sort_column_id(col,COL_TREELIST_MILLESIME);
	
	/* --- TreeList Column #2 --- */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, _("Qty"));
	gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList), col);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func(col, renderer, maturite_cell_data_function, NULL, NULL);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_TREELIST_QTY);
	gtk_tree_view_column_set_sort_column_id(col,COL_TREELIST_QTY);
	
	/* --- TreeList Column #3 --- */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, _("Domain"));
	gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList), col);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func(col, renderer, maturite_cell_data_function, NULL, NULL);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_TREELIST_DOMAIN);
	gtk_tree_view_column_set_sort_column_id(col,COL_TREELIST_DOMAIN);
	
	/* --- TreeList Column #4 -- COUNTRY --- */
        renderer = gtk_cell_renderer_text_new();
        col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREELIST_COUNTRY,NULL);
        gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList), col);
	gtk_tree_view_column_set_visible (col,FALSE);
	
	/* --- TreeList Column #5 -- REGION --- */
        renderer = gtk_cell_renderer_text_new();
        col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREELIST_REGION,NULL);
        gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList), col);
	gtk_tree_view_column_set_visible (col,FALSE);
	
	/* --- TreeList Column #6 -- APPELLATION --- */
        renderer = gtk_cell_renderer_text_new();
        col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREELIST_APPELLATION,NULL);
        gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList), col);
	gtk_tree_view_column_set_visible (col,FALSE);
	
	/* --- TreeList Column #7 -- COLOR --- */
        renderer = gtk_cell_renderer_text_new();
        col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREELIST_COLOR,NULL);
        gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList), col);
	gtk_tree_view_column_set_visible (col,FALSE);
	/* --- TreeList Column #8 -- MATURITE --- */
        renderer = gtk_cell_renderer_text_new();
        /*
	 * 2.2.6
	 * col = gtk_tree_view_column_new_with_attributes("label",renderer,"gint", COL_TREELIST_MATURITE,NULL);
	 * warning lors de l execution 
	 * GLib-GObject-WARNING **: IA__g_object_set_property: object class `GtkCellRendererText' has no property named `gint'
	 */
        col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREELIST_MATURITE,NULL);
        gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList), col);
	gtk_tree_view_column_set_visible (col,FALSE);
	
	/* --- TreeList Column #9 -- GList data --- */
        renderer = gtk_cell_renderer_text_new();
	/*
	 * 2.2.6
	 *  GLib-GObject-WARNING **: unable to set property `text' of type `gchararray' from value of type `gpointer'
	 * col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREELIST_PDATA,NULL);
	 * gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList), col);
	 * gtk_tree_view_column_set_visible (col,FALSE);
	 */
	
	/* --- TreeList Column #10 -- GList --- */
        renderer = gtk_cell_renderer_text_new();
	/*
	 * 2.2.6
	 *  GLib-GObject-WARNING **: unable to set property `text' of type `gchararray' from value of type `gpointer'
	 * col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREELIST_PGLIST,NULL);
	 * gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList), col);
	 * gtk_tree_view_column_set_visible (col,FALSE);
	 */

	/* destroy model automatically with view */
	g_object_unref(StoreView);
	g_object_unref(StoreList);
	
	/*
	 * signal for doube click management
	 */
	g_signal_connect ((gpointer) TreeList, "row_activated",
					G_CALLBACK (on_TreeList_row_activated),
					NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT (window_xcave, scrolledwindow1, "scrolledwindow1");
	GLADE_HOOKUP_OBJECT (window_xcave, TreeView, "TreeView");
	GLADE_HOOKUP_OBJECT (window_xcave, scrolledwindow2, "scrolledwindow2");
	GLADE_HOOKUP_OBJECT (window_xcave, TreeList, "TreeList");

	window_detail_origine = NO_BUTTON;
	
    XCAVE_DEBUG_E;
}

void
quick_message (gchar * message)
{

    /* desactive window_xcave */
    gtk_widget_set_sensitive(window_detail, FALSE);

    GtkWidget	*dialog;
    dialog = gtk_message_dialog_new ((GtkWindow *)window_detail,
                                     GTK_DIALOG_DESTROY_WITH_PARENT,
                                     GTK_MESSAGE_WARNING,
                                     GTK_BUTTONS_CLOSE,
                                     message);
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);

    gtk_widget_set_sensitive(window_detail, TRUE);
}

/*
 * from TreeList a row is selected - doubleclick or button_modif -
 * list_stock_rowSelect contains pointer to wine informations row in list_stock
 * display of detail_window 
 */
void
on_xcave_window_display_modif (	Vin	*Vin_rowSelected)
{
	GList        *tmp_list_country_region        = NULL;
	GList        *tmp_list_region_appellation = NULL;
	
	Purchase     *tmpPurchase;
	GList        *tmp_list_purchase;
	GList        *list_country_screen;
	GList        *list_region_combo;
	GList        *list_appellation_combo;
	
	Drinking     *tmpDrinking;
	GList        *tmp_list_drinking;
	
	gchar          ach[purchaseDate_size];
	gchar          an[vintage_size], cons[potentielDeGarde_size];
	
	GtkWidget    *statv;
	gchar          statxt[80];
	gint	      i;
  

    XCAVE_DEBUG_B;

	fixed=gdk_font_load("-misc-fixed-medium-*-*-*-10-*-*-*-*-*-*-*");

	/* initialisation du compteur 'Nombre de bouteille detail' */
	NbBottleDetail = 0;
	
	window_detail = create_window_detail ();
	
	/*
	 * redimensionnement de la fenetre
	 */
	if ( strlen(WindowsDetailHeight) != 0 && strlen (WindowsDetailWidth) != 0 ) {
	   gtk_window_set_default_size (GTK_WINDOW (window_detail),
					atoi(WindowsDetailWidth),
					atoi(WindowsDetailHeight));
	}
	
	gtk_widget_show_all (window_detail);
	
	statv = lookup_widget (window_detail, "StatusbarDetail");
	ctxtid_detail = gtk_statusbar_get_context_id ((GtkStatusbar *)statv, "default");
	statid_detail = 0;
	
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "vinnom")),       Vin_rowSelected->Nom);
	// general
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "vinprod")),      Vin_rowSelected->supplier);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "vinprodurl")),   Vin_rowSelected->wineMakerAdr);
	sprintf (an, "%d", Vin_rowSelected->vintage);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "vinan")),        (gchar *) an);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "vincoulentry")), Vin_rowSelected->color);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "vincepentry")),  Vin_rowSelected->Cepage);
	sprintf (cons, "%d", Vin_rowSelected->Conserv);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "vincons")),      (gchar *) cons);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "vintemp")),      Vin_rowSelected->temperatureDeDegustation);
	
	// stock
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_qty")), (guint) Vin_rowSelected->qty);

	/* 
	 * for the stock - let's go
	 */
	xcave_createRack(window_detail, Vin_rowSelected->list_emplacement);

	// Purchase
	GtkWidget		*treeList_purchase;
	GtkListStore		*StoreList_purchase;
	GtkTreeIter 		pIter;
	
	xcave_create_TreeList_purchase();
	
	treeList_purchase    = lookup_widget (window_detail, "treelist_purchase");	
	StoreList_purchase   = (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)treeList_purchase);

	tmpPurchase = g_malloc (sizeof(Purchase));
	/*
	 * 2.2.6
	 * Pango-WARNING **: Invalid UTF-8 string passed to pango_layout_set_text()
	 */
	memset(tmpPurchase,'\0',sizeof(Purchase));
	
	for(tmp_list_purchase = Vin_rowSelected->list_purchase;
			tmp_list_purchase != NULL; 
			tmp_list_purchase = tmp_list_purchase->next) {
		memset(tmpPurchase,'\0',sizeof(Purchase));
		memcpy(tmpPurchase,tmp_list_purchase->data,sizeof(Purchase));

		gtk_list_store_append(StoreList_purchase, &pIter);
		gtk_list_store_set(StoreList_purchase, &pIter,
					COL_TREELIST_PURCHASE_DATE, (guint)tmpPurchase->Achat,
					COL_TREELIST_PURCHASE_QTY, (guint)tmpPurchase->qty,
					COL_TREELIST_PURCHASE_PRICE, (gfloat)tmpPurchase->Prix,
					COL_TREELIST_SUPPLIER, &tmpPurchase->Supplier,
					COL_TREELIST_SUPPLIER_ADR, &tmpPurchase->SupplierAdr,
									-1);			

	   NbBottleDetail += tmpPurchase->qty;
	}
	g_free(tmpPurchase);

	g_object_unref(StoreList_purchase);

	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_price")), (gfloat) 0);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_qty")), (guint) 1);
	sprintf (ach, "%d", nowan);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "entry_purchase_year")),       (gchar *) ach);

	// drinking
	GtkWidget		*treeList_drinking;
	GtkListStore		*StoreList_drinking;

	xcave_create_TreeList_drinking();

	treeList_drinking = lookup_widget (window_detail, "treelist_drinking");	
	StoreList_drinking	= (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)treeList_drinking);

	tmpDrinking = g_malloc (sizeof(Drinking));
	
	for(tmp_list_drinking = Vin_rowSelected->list_drinking; 
		tmp_list_drinking != NULL; 
		tmp_list_drinking = tmp_list_drinking->next) {

                GdkPixbuf     *icon;
                GError        *error = NULL;

		memset(tmpDrinking,'\0',sizeof(Drinking));
		memcpy(tmpDrinking,tmp_list_drinking->data,sizeof(Drinking));

	        if (tmpDrinking->degustation_sheet_id != (time_t)0) 
	           icon = create_pixbuf("xcave_degustation.png");
	        else 
	           icon = gdk_pixbuf_new_from_file("", &error);
		gtk_list_store_append(StoreList_drinking, &pIter);
		gtk_list_store_set(StoreList_drinking, &pIter,
                                        COL_TREELIST_ICON,icon,
                                        COL_TREELIST_DRINKING_DATE, (guint)tmpDrinking->Conso,
                                        COL_TREELIST_DRINKING_QTY, (guint)tmpDrinking->qty,
                                        COL_TREELIST_NOTE, tmpDrinking->Note,
                                        COL_TREELIST_COMMENTAIRE, tmpDrinking->Commentaire,
                                        COL_TREELIST_SHEET_ID,tmpDrinking->degustation_sheet_id,
                                        -1);			

	   NbBottleDetail -= tmpDrinking->qty;
	}

	g_free(tmpDrinking);
	g_object_unref(StoreList_drinking);

	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "entry_drinking_year")),       (gchar *) ach);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_drinking_qty")), (guint) 1);

	sprintf (statxt, _("Number of bottles for the selection : %d"), NbBottleDetail);
	// gestion du nombre de bouteille pour le vin selectionn 
	if (statid_detail != 0) gtk_statusbar_remove ((GtkStatusbar *)statv, ctxtid_detail, statid_detail);
	statid_detail = gtk_statusbar_push ((GtkStatusbar *)statv, ctxtid_detail, statxt);
	
	// commentaire
	GtkWidget		*vincom;
	GtkTextBuffer	*buffer_vincom;
	
	vincom 	= lookup_widget (window_detail, "vincom");
	buffer_vincom = gtk_text_view_get_buffer (GTK_TEXT_VIEW (vincom));/* C */
	
       /*
	* commentaire. Remplacement de tout les carateres  par <CR> avant affichage.
	* le caractere <CR> indique un fin d'enregistrement dans un fichier stream.
	*/
	for ( i=0 ; i < (comment_size - 1) ; i++) 
		if (strncmp(&Vin_rowSelected->Commentaire[i], "",1) == 0) 
			strncpy(&Vin_rowSelected->Commentaire[i],"\n",1);

 	gtk_text_buffer_set_text (buffer_vincom, Vin_rowSelected->Commentaire, -1);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "vinnote")), Vin_rowSelected->Note);
					
	/* PAYS */
	list_country_screen = g_list_copy(list_country);
	if (list_country_screen != NULL)
		gtk_combo_set_popdown_strings (GTK_COMBO (lookup_widget (window_detail, "wineCountryList")), 
										list_country_screen);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "wineCountryEntry")), 
						Vin_rowSelected->country);

	/* REGION */
	list_region_combo = NULL;
	for(tmp_list_country_region = list_country_region; 
		tmp_list_country_region != NULL; 
		tmp_list_country_region = tmp_list_country_region->next) {
		  if (strcasecmp(tmp_list_country_region->data, Vin_rowSelected->country) == 0) 
		  list_region_combo = g_list_append(list_region_combo,tmp_list_country_region->data + country_size);
	}
	if (g_list_length(list_region_combo) == 0)
		list_region_combo = g_list_append(list_region_combo,"");

	gtk_combo_set_popdown_strings (GTK_COMBO (lookup_widget (window_detail, "vinreglist")), list_region_combo);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "vinregentry")), Vin_rowSelected->Region);

	/* APPELLATION / TERROIR */
	list_appellation_combo = NULL;
	for(tmp_list_region_appellation = list_region_appellation; 
		tmp_list_region_appellation != NULL; 
		tmp_list_region_appellation = tmp_list_region_appellation->next) {
		/*
		  if (( strcasecmp(tmp_list_region_appellation->data, Vin_rowSelected->Region) == 0)
			  ||
			 ( strcasecmp(tmp_list_region_appellation->data, Vin_rowSelected->country) == 0)
		  )
		 */
	  if ( strcasecmp(tmp_list_region_appellation->data, Vin_rowSelected->Region) == 0)
		list_appellation_combo = g_list_append(list_appellation_combo,tmp_list_region_appellation->data + region_size);
	}

	if (g_list_length(list_appellation_combo) == 0) 
		list_appellation_combo = g_list_append(list_appellation_combo,"");

	gtk_combo_set_popdown_strings (GTK_COMBO (lookup_widget (window_detail, "vinaplist")), list_appellation);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "vinapentry")), Vin_rowSelected->Appellation);

	/* CEPAGE
	 */
	if (list_cepage != NULL)
		gtk_combo_set_popdown_strings (GTK_COMBO (lookup_widget (window_detail, "vinceplist")), list_cepage);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "vincepentry")), Vin_rowSelected->Cepage);
	
	/*
	 * armememnt de trois signaux dont on souhaite conserver l'id pour un blocage
	 * ulterieur
	 */
	xcave_signal_connect();

    XCAVE_DEBUG_E;
}

void
on_xcave_window_modif_clicked (void)
{
  
	Vin			  *Vin_rowSelected;

    XCAVE_DEBUG_B;

	/*
	 * we need selected data 
	 */
	Vin_rowSelected = xcave_id_selected_treelist_row();

	if (Vin_rowSelected != NULL) {
		/* 
		 * desactive window_xcave
		 */
		gtk_widget_set_sensitive(window_xcave, FALSE);
	
		window_detail_origine = BUTTON_MODIF;
	
		on_xcave_window_display_modif(Vin_rowSelected);
	}

    XCAVE_DEBUG_E;
}

/*
 * GtkWindow "xcave_window".
 * button_add clicked
 */
void
on_xcave_window_add_button_clicked (GtkButton * button,
		   gpointer user_data)
{
	GtkCombo *reglist = NULL, *applist = NULL, *countryList = NULL, *ceplist = NULL;
	gchar tmpreg[region_size];
	gchar tmpCountry[country_size];
	gchar ach[purchaseDate_size];

    XCAVE_DEBUG_B;

	/*
	 * 2.2.6
	 */
        NbBottleDetail = 0;

	strncpy (tmpreg, "",region_size);
	strncpy (tmpCountry, "",country_size);

	if (window_detail_origine != BUTTON_RACK_ADD)
		window_detail_origine = BUTTON_ADD;
	
	/* 
	 * desactive window_xcave
	 */
	gtk_widget_set_sensitive(window_xcave, FALSE);
	
	window_detail = create_window_detail ();
	
	/*
	 * armememnt de trois signaux dont on souhaite conserver l'id pour un blocage
	 * ulterieur
	 */
	xcave_signal_connect();
	
	/*
	 * redimensionnement de la fenetre
	 */
	if ( strlen(WindowsDetailHeight) != 0 && strlen (WindowsDetailWidth) != 0 ) {
	   gtk_window_set_default_size (GTK_WINDOW (window_detail), atoi(WindowsDetailWidth), atoi(WindowsDetailHeight));
	}
	
	gtk_widget_show (window_detail);
	
	countryList = GTK_COMBO (lookup_widget (window_detail, "wineCountryList"));
	reglist  = GTK_COMBO (lookup_widget (window_detail, "vinreglist"));
	applist  = GTK_COMBO (lookup_widget (window_detail, "vinaplist"));
	ceplist  = GTK_COMBO (lookup_widget (window_detail, "vinceplist"));
	
	gtk_combo_set_popdown_strings (GTK_COMBO (countryList), list_country);
	gtk_combo_set_popdown_strings (GTK_COMBO (reglist),  list_region);
	gtk_combo_set_popdown_strings (GTK_COMBO (applist),  list_appellation);
	gtk_combo_set_popdown_strings (GTK_COMBO (lookup_widget (window_detail, "vinceplist")), list_cepage);
	
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_qty")), (gfloat) 1);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_drinking_qty")), (gfloat) 1);
	
	sprintf (ach, "%d", nowan);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "entry_purchase_year")),      (gchar *) ach);
	gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_drinking_year"))     ,(gchar *) ach);
	
	xcave_create_TreeList_purchase(); 
	
	xcave_create_TreeList_drinking();
	
	/* 
	 * for the stock - let's go
	 */
	xcave_createRack(window_detail, NULL);

    XCAVE_DEBUG_E;
}

/*
 *	
 *
 */
void
on_window_detail_button_ok_clicked (GtkButton * button,
									gpointer user_data)
{
	Vin          	*tmpVin;
	Vin             *Vin_rowSelected;
	ptrCateg        ptmpCtg;

    XCAVE_DEBUG_B;

	tmpVin = g_malloc ( sizeof(Vin) );
	memset(tmpVin, '\0', sizeof( Vin ));

	strcpy (tmpVin->Nom, gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vinnom"))));

	/* get data from vpaned 'general' */
	strcpy (tmpVin->supplier, gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vinprod"))));
	strcpy (tmpVin->wineMakerAdr,	gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vinprodurl"))));
	tmpVin->vintage = atoi (gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vinan"))));
	strcpy (tmpVin->color, gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vincoulentry"))));
	strcpy (tmpVin->country, gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "wineCountryEntry"))));
	strcpy (tmpVin->Region, gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vinregentry"))));
	strcpy (tmpVin->Appellation, gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vinapentry"))));
	strcpy (tmpVin->Cepage, gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vincepentry"))));
	tmpVin->Conserv   = atoi (gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vincons"))));
	strcpy (tmpVin->temperatureDeDegustation, gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vintemp"))));
	

	/*
	 * get data from vpaned 'racks'
	 */
	tmpVin->qty = 0;
	tmpVin->list_emplacement = xcave_rackTreatment(tmpVin->Nom);
	
	/*
	 * get data from vpaned 'purchase'
	 */
	GtkWidget		*TreeList_purchase;
	GtkTreeModel	*treelist_purchase_model;
	GtkTreeIter 	treelist_iter;
	gboolean		btreelist = TRUE;
	Purchase		*tmp_purchase;
	
	TreeList_purchase = lookup_widget(window_detail,"treelist_purchase");
	treelist_purchase_model	= gtk_tree_view_get_model((GtkTreeView *)TreeList_purchase);

	btreelist = gtk_tree_model_get_iter_first(treelist_purchase_model,/* C */
											  &treelist_iter);
	   												
	while (btreelist == TRUE) {
		gchar	*pSupplier;
		gchar	*pSupplierAdr;
	
		tmp_purchase = g_malloc(sizeof(Purchase));
		pSupplier = g_malloc(supplier_size);
		pSupplierAdr = g_malloc(supplierAdr_size);
	        /*
	         * 2.2.6
        	 * Pango-WARNING **: Invalid UTF-8 string passed to pango_layout_set_text()
        	 */
        	memset(tmp_purchase,'\0',sizeof(Purchase));
		memset(pSupplier,'\0',supplier_size);
		memset(pSupplierAdr,'\0',supplierAdr_size);
	
		gtk_tree_model_get (treelist_purchase_model, &treelist_iter,
							COL_TREELIST_PURCHASE_DATE, &tmp_purchase->Achat,
							COL_TREELIST_PURCHASE_QTY, &tmp_purchase->qty,
							COL_TREELIST_PURCHASE_PRICE, &tmp_purchase->Prix,
							COL_TREELIST_SUPPLIER, &pSupplier,
							COL_TREELIST_SUPPLIER_ADR, &pSupplierAdr,
							-1);
	
		strcpy(tmp_purchase->Supplier,pSupplier);
		strcpy(tmp_purchase->SupplierAdr,pSupplierAdr);
		g_free(pSupplier);
		g_free(pSupplierAdr);
		
		tmpVin->list_purchase = g_list_append(tmpVin->list_purchase, tmp_purchase);
	
		btreelist = gtk_tree_model_iter_next(treelist_purchase_model,
                                             &treelist_iter);
		tmpVin->qty += tmp_purchase->qty;
	}
											  
	/*
	 * get data from vpaned 'drinking'
	 */
	GtkWidget		*TreeList_drinking;
	GtkTreeModel	*treelist_drinking_model;
	Drinking		*tmp_drinking;

	TreeList_drinking = lookup_widget(window_detail,"treelist_drinking");
	treelist_drinking_model	= gtk_tree_view_get_model((GtkTreeView *)TreeList_drinking);

	btreelist = gtk_tree_model_get_iter_first(treelist_drinking_model, &treelist_iter);
	
	while (btreelist == TRUE) {

		gchar	*pNote;
		gchar	*pCommentaire;
	
		pNote = g_malloc(rated_size);
		pCommentaire = g_malloc(comment_size);
		tmp_drinking = g_malloc(sizeof(Drinking));
	        memset(tmp_drinking, '\0', sizeof( Drinking ));
	
		gtk_tree_model_get (treelist_drinking_model, &treelist_iter,
                                    COL_TREELIST_DRINKING_DATE, &tmp_drinking->Conso,
                                    COL_TREELIST_DRINKING_QTY, &tmp_drinking->qty,
                                    COL_TREELIST_NOTE, &pNote,
                                    COL_TREELIST_COMMENTAIRE, &pCommentaire,
                                    COL_TREELIST_SHEET_ID,&tmp_drinking->degustation_sheet_id,
                                    -1);
	
                xcave_debug_2str(__FUNCTION__,"degustation-sheet-id",25,ctime((time_t *)&tmp_drinking->degustation_sheet_id));

		strcpy(tmp_drinking->Note, pNote);
		strcpy(tmp_drinking->Commentaire, pCommentaire);
		g_free(pNote);
		g_free(pCommentaire);
	
		tmpVin->list_drinking = g_list_append(tmpVin->list_drinking, tmp_drinking);
	
		btreelist = gtk_tree_model_iter_next(treelist_drinking_model, &treelist_iter);
		tmpVin->qty -= tmp_drinking->qty;

		/*
		 * degustation sheet 
		 */
		GList              *tmp_list_ds;
		degustation_sheet  *tmp_ds;

                for(tmp_list_ds = list_ds; tmp_list_ds != NULL; tmp_list_ds = tmp_list_ds->next) {
                   tmp_ds = g_malloc ( sizeof(degustation_sheet) );
                   memset(tmp_ds,'\0',sizeof(degustation_sheet));
                   memcpy(tmp_ds,tmp_list_ds->data,sizeof(degustation_sheet));
                   if (tmp_ds->degustation_sheet_id == tmp_drinking->degustation_sheet_id) {
                      tmpVin->list_degustation = g_list_append(tmpVin->list_degustation, tmp_ds);
		      /*
		       * if the degustation sheet exist before, it must be suppress before.
		       * The ds can exist if the drinking line was modified and not created.
		       */
		       GList              *tmp_list_full_ds;
		       degustation_sheet  *tmp_full_ds;
                       tmp_full_ds = g_malloc ( sizeof(degustation_sheet) );
                       for(tmp_list_full_ds = list_full_ds; tmp_list_full_ds != NULL; tmp_list_full_ds = tmp_list_full_ds->next) {
                          memset(tmp_full_ds,'\0',sizeof(degustation_sheet));
                          memcpy(tmp_full_ds,tmp_list_full_ds->data,sizeof(degustation_sheet));
                          if (tmp_full_ds->degustation_sheet_id == tmp_drinking->degustation_sheet_id) {
                              list_full_ds = g_list_remove_link(list_full_ds,tmp_list_full_ds);
			      g_list_free1(tmp_list_full_ds);
                              xcave_debug_msg_int(__FUNCTION__,"g_list_remove(list_full_ds) ok",5,g_list_length(list_full_ds));
			      break;
                          } else {
                              xcave_debug_msg_int(__FUNCTION__,"g_list_remove(list_full_ds) ko",5,g_list_length(list_full_ds));
                          }
		       }
                      g_free(tmp_full_ds);
                      list_full_ds = g_list_append(list_full_ds, tmp_ds);
		   }
		   else
                      g_free(tmp_ds);
	           xcave_debug_msg_int(__FUNCTION__,"list_full_ds",2,g_list_length(list_full_ds));
		}

	}
	
	/*
	 * get data from vpaned 'comment'
	 */
	GtkWidget		*vincom;
	GtkTextBuffer	*buffer_vincom;
	GtkTextIter		start_vincom;
	GtkTextIter 	end_vincom;
	gchar			*comment;
	vincom 	= lookup_widget (window_detail, "vincom");
	buffer_vincom = gtk_text_view_get_buffer (GTK_TEXT_VIEW (vincom));

	gtk_text_buffer_get_start_iter(buffer_vincom,&start_vincom);
	gtk_text_buffer_get_end_iter(buffer_vincom,&end_vincom);
	
	comment = g_malloc(gtk_text_buffer_get_char_count(buffer_vincom) );
	comment = gtk_text_buffer_get_text(buffer_vincom, &start_vincom, &end_vincom, FALSE );

	strcpy(tmpVin->Commentaire,comment);
	g_free(comment);
	
	/*
	 * commentaire. Remplacement de tout les carateres <CR> par  avant mise  jour du fichier stocks.
	 * le caractere <CR> indique un fin d'enregistrement dans un fichier stream.
	 */
	gint i;
	for ( i=0 ; i < (comment_size - 1) ; i++) 
		if (strncmp(&tmpVin->Commentaire[i], "\n",1) == 0) strncpy(&tmpVin->Commentaire[i],"",1);

	/*
	 * drinking wine rate ... or whatever u want
	 */
	strcpy (tmpVin->Note,gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vinnote"))));

	/*
	 * coherency controls
	 */
	gboolean	check_ok = TRUE;
	
	if ( strlen (tmpVin->Nom)	== 0 && check_ok == TRUE ) {
		quick_message (_(" Wine name's missing "));
		check_ok = FALSE;
	}
	if (strlen (tmpVin->color) == 0 && check_ok == TRUE ) {
		quick_message (_(" Color's missing "));
		check_ok = FALSE;
	}
	if ( (tmpVin->vintage < 1800 || tmpVin->vintage > nowan) && check_ok == TRUE ) {
		quick_message (_(" Vintage year's error "));
		check_ok = FALSE;
	}
	if ( tmpVin->Conserv < 0 && check_ok == TRUE )  {
		quick_message (_(" Data number of year for conservation is false "));
		check_ok = FALSE;
	}
	if ( strlen (tmpVin->country)	== 0 && check_ok == TRUE ) {
		quick_message (_(" Country's missing "));
		check_ok = FALSE;
	}
	if ( strlen (tmpVin->Region)	== 0 && check_ok == TRUE ) {
		quick_message (_(" Region's missing "));
		check_ok = FALSE;
	}
	if ( strlen (tmpVin->Appellation) == 0 && check_ok == TRUE ) {
		quick_message (_(" Appellation's missing "));
		check_ok = FALSE;
	}

	/*
	 * gestion des doublons
	 */
	if ( (window_detail_origine == BUTTON_ADD || window_detail_origine == BUTTON_RACK_ADD)
		&&
		 (g_list_find_custom(list_stock,(gpointer) tmpVin,(GCompareFunc)find_existing_wine) != NULL) 
		&&
		 (check_ok == TRUE)
		) {
		quick_message (_("A wine with similar characteristic\n\tcountry\n\tregion\n\tappellation\n\tdomain\n\tvintage is already in the cellar"));
		check_ok = FALSE;
	}

	if (tmpVin->qty < g_list_length(tmpVin->list_emplacement) && check_ok == TRUE ) {
		quick_message (_(" More bottles in Emplacement than in the stocks "));
		check_ok = FALSE;
	} 
	if (tmpVin->qty < 0 && check_ok == TRUE ) {
		quick_message (_(" You drink too much (more bottles drinking than purchase ! "));
		check_ok = FALSE;
	} 

	if ( 	check_ok == TRUE ) {

		 /* controle de la relation region/appellation 
			for(tmp_list_appellation = list_appellation; 
				tmp_list_appellation != NULL; 
				tmp_list_appellation = tmp_list_appellation->next) {
				  memcpy(&tmpApp,tmp_list_appellation->data,appellation_size);
				  if (  (  strcmp (data->Appellation, tmpApp.Appellation) == 0 
						&& strcmp (data->Region     , tmpApp.Region     ) != 0) 
					 || (  strcmp (data->Appellation, tmpApp.Appellation) == 0 
						&& strcmp (data->Region     , tmpApp.Region     ) != 0))
						   quick_message (_(" No such appellation for this region "));
		}
		if (tmp_list_appellation == (g_list_last(list_appellation)))
		{
		}
		*/

		/* Positionnement du flag 'maturite' */
        if (tmpVin->qty == 0)
     	    tmpVin->Maturite = -2;
        else
            if (tmpVin->vintage ==0 || tmpVin->Conserv == 0) 
	       tmpVin->Maturite = -1;
            else
               tmpVin->Maturite=(100*(nowan-(tmpVin->vintage))/tmpVin->Conserv);

		switch(window_detail_origine) {
			case (BUTTON_ADD): /* creation d un nouveau vin*/  
				list_stock = g_list_append(list_stock ,tmpVin);
				list_stock = g_list_sort(list_stock,(GCompareFunc)cmp_stock);
		                window_detail_origine = NO_BUTTON;
                                /*
                                 * need to clean the list_ds
                                 */
                                 xcave_debug_msg_int(__FUNCTION__,"list_ds",2,g_list_length(list_ds));
                                 list_ds = xcave_g_list_free(list_ds);
                                 xcave_debug_msg_int(__FUNCTION__,"list_ds",2,g_list_length(list_ds));
                                 break;
				
			case (BUTTON_RACK_ADD): /* creation d un nouveau vin*/
				list_stock = g_list_append(list_stock ,tmpVin);
				list_stock = g_list_sort(list_stock,(GCompareFunc)cmp_stock);
				/* window should be closed, list_append_rack should free data 
				   used by window_rack. We could possibly imagine open again
				   window_rack ... or redesign large amount of xcave (-; */
		                window_detail_origine = BUTTON_RACK;
				break;
	
			case (BUTTON_MODIF): /* modification d'un vin deja existant */
				/* identifier le vin qui a ete modifie */
	
				/* we need selected data  */
				Vin_rowSelected = xcave_id_selected_treelist_row();
	
				/*
				 * BEWARE - update of the data straight in the list_stock
				 * 			get a pointer on the glist element from COL_TREELIST_PDATA
				 *			of GtkTreeList TreeList
				 */
			
				/* Emplacement */
				/* liberer la liste precedente avant svp */
				Vin_rowSelected->list_emplacement = xcave_g_list_free(Vin_rowSelected->list_emplacement);
		
				/* Purchase */
				/* liberer la liste precedente avant svp */
				Vin_rowSelected->list_purchase = xcave_g_list_free(Vin_rowSelected->list_purchase);
				
				/* Drinking */
				/* liberer la liste precedente avant svp */
				Vin_rowSelected->list_drinking = xcave_g_list_free(Vin_rowSelected->list_drinking);
				
				list_stock = g_list_remove(list_stock, Vin_rowSelected);
				list_stock = g_list_append(list_stock ,tmpVin);
	
		                window_detail_origine = NO_BUTTON;
				break;
				
			case (BUTTON_RACK_MODIF):
				/* identifier le vin qui a ete modifie */
	
				/* we need selected data  */
				Vin_rowSelected = xcave_id_selected_toggledbutton();

				/* Emplacement */
				/* liberer la liste precedente avant svp */
				Vin_rowSelected->list_emplacement = xcave_g_list_free(Vin_rowSelected->list_emplacement);
		
				/* Purchase */
				/* liberer la liste precedente avant svp */
				Vin_rowSelected->list_purchase = xcave_g_list_free(Vin_rowSelected->list_purchase);
				
				/* Drinking */
				/* liberer la liste precedente avant svp */
				Vin_rowSelected->list_drinking = xcave_g_list_free(Vin_rowSelected->list_drinking);
	
				list_stock = g_list_remove(list_stock, Vin_rowSelected);
				list_stock = g_list_append(list_stock ,tmpVin);
				
				/* window should be closed, list_append_rack should free data 
				   used by window_rack. We could possibly imagine open again
				   window_rack ... or redesign large amount of xcave (-; */
		                window_detail_origine = BUTTON_RACK;
				break;
				
			case (BUTTON_DUPLICATE): /* creation d un nouveau vin*/  
				list_stock = g_list_append(list_stock ,tmpVin);
				list_stock = g_list_sort(list_stock,(GCompareFunc)cmp_stock);

		                window_detail_origine = NO_BUTTON;
				break;
				
			case (BUTTON_RACK_DUPLICATE): /* creation d un nouveau vin*/
				list_stock = g_list_append(list_stock ,tmpVin);
				list_stock = g_list_sort(list_stock,(GCompareFunc)cmp_stock);
				/* window should be closed, list_append_rack should free data 
				   used by window_rack. We could possibly imagine open again
				   window_rack ... or redesign large amount of xcave (-; */
		                window_detail_origine = BUTTON_RACK;
				break;
	
			default:
				break;
		}

		list_stock = g_list_sort(list_stock,(GCompareFunc)cmp_stock);
		xcave_xmlWrite(DBPATH);

		/*
		 * need to update storeview and storelist
		 */
		GtkWidget       *treelist;
		GtkWidget       *treeview;
		GtkListStore	*storelist;
		GtkTreeStore	*storeview;
	
		treelist = lookup_widget(window_xcave,"TreeList");
		storelist  = (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)treelist);
		gtk_list_store_clear(storelist);

		treeview = lookup_widget(window_xcave,"TreeView");
		storeview	= (GtkTreeStore *) gtk_tree_view_get_model((GtkTreeView *)treeview);
		gtk_tree_store_clear(storeview);
	    
		xcave_fill_model (storeview, storelist);

		/*
		 * penser  mettre  jour la liste emplacement
		 */
		list_append_rack();
	
		ptmpCtg = g_malloc (sizeof(Categ));
		memset(ptmpCtg, '\0', sizeof( Categ ));
		strcpy(ptmpCtg->color,     tmpVin->color );
		strcpy(ptmpCtg->country,        tmpVin->country );
		strcpy(ptmpCtg->Region,      tmpVin->Region );
		strcpy(ptmpCtg->Appellation, tmpVin->Appellation );
	
		if (g_list_find_custom(list_categ,(gpointer) ptmpCtg,(GCompareFunc)find_categ) == NULL) {
			list_append_categ();
		}
	
		gtk_object_destroy (GTK_OBJECT (window_detail));

		if (window_detail_origine == BUTTON_RACK) {
	           GtkButton *button;
                   gpointer         user_data;

		   gtk_object_destroy (GTK_OBJECT (window_rack));
		   on_xcave_window_button_rack_clicked(button, user_data);
		}
		else
		   gtk_widget_set_sensitive(window_xcave, TRUE);

        }
	else {
		xcave_debug_msg(__FUNCTION__,"on_window_detail_button_ok_clicked:clean up the mess:\n");
		/* 
		 * suite  erreur de controle 
		 * liberation de la memoire allouee
		 */
		xcave_g_list_free(tmpVin->list_emplacement);
		xcave_g_list_free(tmpVin->list_drinking);
		xcave_g_list_free(tmpVin->list_purchase);
		g_free(tmpVin);
	}
    XCAVE_DEBUG_E;
}

/* 
 * GtkWindow 'detail_window'
 * button_add_cancel clicked
 */
void
on_window_detail_button_cancel_clicked (GtkButton * button,
		      gpointer user_data)
{
    XCAVE_DEBUG_B;


	if (window_detail_origine == BUTTON_ADD || window_detail_origine == BUTTON_MODIF) {
		gtk_widget_set_sensitive(window_xcave, TRUE);
		window_detail_origine = NO_BUTTON;
	}
	else {
		gtk_widget_set_sensitive(window_rack, TRUE);
		/* we need selected data for gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggledbutton),FALSE); */
		Vin		*Vin_rowSelected;
		Vin_rowSelected = xcave_id_selected_toggledbutton();
		window_detail_origine = BUTTON_RACK;
	}

	gtk_object_destroy (GTK_OBJECT (window_detail));

	/*
	 * need to clean the list_ds (temporary degustation list)
	 */
        if (lDS != NULL) {
	   xcave_debug_msg(__FUNCTION__,"g_free(lDS)");
           // g_free(lDS);
           // lDS = NULL;
	}

	xcave_debug_msg_int(__FUNCTION__,"xcave_g_list_free(list_ds)",2,g_list_length(list_ds));
	list_ds = xcave_g_list_free(list_ds);

    XCAVE_DEBUG_E;
}

/* 
 * GtkWindow 'detail_window'
 * button_duplicate_clicked
 */
void
on_window_detail_button_duplicate_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
    XCAVE_DEBUG_B;
	gtk_window_set_title(GTK_WINDOW (window_detail), _("Duplicate wine"));

	if (window_detail_origine == BUTTON_RACK_MODIF)
	  window_detail_origine = BUTTON_RACK_DUPLICATE;
	else
	  window_detail_origine = BUTTON_DUPLICATE;
    XCAVE_DEBUG_E;
}
/*
 * GtkWindow 'detail'
 * destroy signal
 */
void
on_window_detail_destroy (GtkObject * object, gpointer user_data)
{
    XCAVE_DEBUG_B;

	/* 
	 * reactive window_xcave
	 */
	if (window_detail_origine == BUTTON_ADD || window_detail_origine == BUTTON_MODIF || window_detail_origine == BUTTON_DUPLICATE)
		gtk_widget_set_sensitive(window_xcave, TRUE);

    XCAVE_DEBUG_E;
}

/*
 * row selected from TreeView -> filter 
 * TreeList display 
 */
void
on_TreeView_cursor_changed             (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
	GtkTreeModel 		*model;
	GtkTreeSelection	*selection;
	GtkTreeIter			iter;
	GtkTreeIter			child;
	GtkTreeIter			grandson;
	gchar				*pAppellation;

    XCAVE_DEBUG_B;
	pAppellation			= g_malloc(appellation_size);

	g_return_if_fail (GTK_IS_TREE_VIEW (treeview));
	selection = gtk_tree_view_get_selection (treeview);  

	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
		return;

	gtk_tree_model_get (model, &iter, COL_TREEVIEW, &pAppellation, -1);

	/*
	 * Determine level the selected data is 
	 */
	if (gtk_tree_model_iter_children(model, &child, &iter) == FALSE) {
		xcave_fill_filtered_model(pAppellation,COL_TREELIST_APPELLATION);
	}
	else {
		if (gtk_tree_model_iter_children(model, &grandson, &child) == FALSE) {
			xcave_fill_filtered_model(pAppellation,COL_TREELIST_REGION);
		}
		else {
			xcave_fill_filtered_model(pAppellation,COL_TREELIST_COUNTRY);

		}
			
	}

	g_free(pAppellation);

    XCAVE_DEBUG_E;
}

/*
 * Double click on TreeList View -> display
 * GtkWindow "detail".
 */
void
on_TreeList_row_activated              (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data)
{
    XCAVE_DEBUG_B;

	on_xcave_window_modif_clicked();

    XCAVE_DEBUG_E;
}

/*
 * button_modif clicked -> display
 * GtkWindow "detail".
 */
void
on_xcave_window_modif_button_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    XCAVE_DEBUG_B;

	on_xcave_window_modif_clicked();

    XCAVE_DEBUG_E;

}

/*
 * supp_modif clicked -> display
 * GtkWindow "ConSupp".
 */
void
on_xcave_window_supp_button_clicked   (GtkButton       *button,
                                       gpointer         user_data)
{
	Vin		*Vin_rowSelected;
    XCAVE_DEBUG_B;

	/*
	 * we need selected data 
	 */
	Vin_rowSelected = xcave_id_selected_treelist_row();

	if (Vin_rowSelected != NULL) {

		/* 
		 * desactive window_xcave
		 */
		gtk_widget_set_sensitive(window_xcave, FALSE);
	
		confirm = create_window_supp ();
		gtk_widget_show (confirm);
	}
    XCAVE_DEBUG_E;

}

/*
 * GtkWindow "ConSupp".
 * cancel_button clicked
 */
void
on_window_supp_button_supp_cancel_clicked (GtkButton * button,
										   gpointer user_data)
{
  gtk_object_destroy (GTK_OBJECT (confirm));

	/* 
	 * reactive window_xcave
	 */
	gtk_widget_set_sensitive(window_xcave, TRUE);
}

/*
 * GtkWindow "ConSupp".
 * cancel_button clicked
 */
void
on_window_supp_button_supp_ok_clicked (GtkButton * button,
										gpointer user_data)
{
	GtkTreeSelection *selection;
	GtkTreeModel     *model;
	GtkTreeIter       selected_row;
	GtkWidget		 *TreeList;

	GList  *list_stock_rowSelected;
	ptrVin	Vin_rowSelected;

    XCAVE_DEBUG_B;

	TreeList = lookup_widget(window_xcave,"TreeList");
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(TreeList));
	g_assert(gtk_tree_selection_get_mode(selection) == GTK_SELECTION_SINGLE);

        if (gtk_tree_selection_get_selected(selection, &model, &selected_row)) {
           /*
            * update list_stock by removing selected wine
            */
            gtk_tree_model_get (model, &selected_row, 
                                COL_TREELIST_PDATA, &Vin_rowSelected,
                                COL_TREELIST_PGLIST, &list_stock_rowSelected,
                                -1);
           /*
            * suppression des fiches de degustations associees
            */

           list_stock = g_list_remove_link(list_stock,list_stock_rowSelected);
           list_stock = g_list_sort(list_stock,(GCompareFunc)cmp_stock);
           xcave_xmlWrite(DBPATH);

           /*
            * update TreeList
            */
           gtk_list_store_remove(GTK_LIST_STORE(model), &selected_row);
	}

	/*
	 * need to update storeview and storelist
	 */
	GtkWidget		*treelist;
	GtkWidget		*treeview;
	GtkListStore	*storelist;
	GtkTreeStore	*storeview;
	
	treelist = lookup_widget(window_xcave,"TreeList");
	storelist	= (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)treelist);
	gtk_list_store_clear(storelist);

	treeview = lookup_widget(window_xcave,"TreeView");
	storeview	= (GtkTreeStore *) gtk_tree_view_get_model((GtkTreeView *)treeview);
	gtk_tree_store_clear(storeview);
	
	xcave_fill_model (storeview, storelist);

	/*
	 * penser  mettre  jour la liste emplacement
	 */
	list_append_rack();

	gtk_object_destroy (GTK_OBJECT (confirm));
	
	/* 
	 * reactive window_xcave
	 */
	gtk_widget_set_sensitive(window_xcave, TRUE);
	
    XCAVE_DEBUG_E;
}

/*
 * GtkWindow 'xcave'
 * GtkEntry 'color' changed 
 */
void
on_entry_color_changed (GtkEditable * editable,
						gpointer 	user_data)
{
	gchar				*pentry;

    XCAVE_DEBUG_B;

	pentry =(gchar *) gtk_entry_get_text    ((GtkEntry *)editable);
	xcave_fill_filtered_model(pentry,COL_TREELIST_COLOR);

    XCAVE_DEBUG_E;
}
