/*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more detailvins.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <errno.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
	g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
	g_object_set_data (G_OBJECT (component), name, widget)

void
xcave_createRack(GtkWidget *window, GList *vin_list_emplacement)
{

	GtkWidget   *hbox_rack;
	GtkWidget   *scrolledwindow_rack;
	GtkWidget   *viewport_rack;
	GtkWidget   *frameRack[rackMax];
        GtkWidget    *tableRack[rackMax];
	GtkWidget    *labelRackUnit[rackMax][rackRowMax][rackColMax];
	GtkWidget    *checkbutton[rackMax][rackRowMax][rackColMax];
	GtkTooltips  *tooltips;
	GList        *tmp_list_emp;
	GList        *tmp_list_rack;

	gint           rackNum;

    XCAVE_DEBUG_B;

	scrolledwindow_rack = lookup_widget (window, "scrolledwindow_rack");

	tooltips = gtk_tooltips_new ();

	viewport_rack = gtk_viewport_new (NULL, NULL);
	gtk_widget_show (viewport_rack);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_rack), viewport_rack);
	GLADE_HOOKUP_OBJECT (window, viewport_rack, "viewport_rack");
	
	hbox_rack = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox_rack);
	gtk_container_add (GTK_CONTAINER (viewport_rack), hbox_rack);
	GLADE_HOOKUP_OBJECT (window, hbox_rack, "hbox_rack");

	rackNb = g_list_length(list_rackRow);
	for (rackNum=0; rackNum < rackNb; rackNum++) {
		gchar   buf[64];
		gchar   lab[32];

                rackRowNb =(gint)g_list_nth_data(list_rackCol, rackNum);
                rackColNb = (gint)g_list_nth_data (list_rackCol, rackNum);

		/* 
		 * create as much as frame than racks number 
		 */
		sprintf(buf,"frameRack_rack%d",rackNum+1);
		sprintf(lab,_("racks %d"),rackNum+1);
		
		frameRack[rackNum] = gtk_frame_new (lab);
		gtk_widget_set_name (frameRack[rackNum], buf);
		gtk_widget_show (frameRack[rackNum]);
		gtk_box_pack_start (GTK_BOX (hbox_rack), frameRack[rackNum], TRUE, TRUE, 0);
		GLADE_HOOKUP_OBJECT (window, frameRack[rackNum], buf);

		/* 
		 * For each frame create a table 
		 */
		sprintf(buf,"tableRack_rack%d",rackNum+1);
		
		tableRack[rackNum] = gtk_table_new (rackRowNb, rackColNb, TRUE);
		gtk_widget_set_name (tableRack[rackNum], buf);
		gtk_widget_show (tableRack[rackNum]);
		gtk_container_add (GTK_CONTAINER (frameRack[rackNum]), tableRack[rackNum]);
		GLADE_HOOKUP_OBJECT (window, tableRack[rackNum], buf); 
	}

	/*
	 * 
	 */
	for (rackNum=0; rackNum < rackNb; rackNum++) {
	        gint    NumRow;

                rackRowNb =(gint)g_list_nth_data(list_rackRow, rackNum);
                rackColNb = (gint)g_list_nth_data (list_rackCol, rackNum);

		// label ligne
		for (NumRow=1; NumRow < rackRowNb + 1; NumRow++) {
			gint    NumCol;
		
			// label colonne
			for (NumCol=1; NumCol < rackColNb + 1; NumCol++) {
				gchar			buf[64];
				Emplacement		emp;
				gint			trouve = 0;
				Emplacement		tmpEmplacement;
				ptrRack		        tmp_rack;
				ptrVin			tmp_vin;
		
				sprintf(buf,"checkbutton%d-%d-%d",rackNum,NumRow,NumCol);
			
				memset(&emp,'\0',sizeof(Emplacement));
				emp.Rack = rackNum;
				emp.Row    = NumRow;
				emp.Column = NumCol;
		
				/*
				 * recherche de l'emplacement de la(es) bouteilles selectionnees
				 * pour autoriser l'utilisateur a deplacer les bouteilles dans le 
				 * stock l'emplacement n'est pas fige
				 */
				for(tmp_list_emp = vin_list_emplacement; 
					tmp_list_emp != NULL ; 
					tmp_list_emp = tmp_list_emp->next) {
						memcpy(&tmpEmplacement,
								tmp_list_emp->data,
								sizeof(Emplacement));
						if (  rackNum == tmpEmplacement.Rack
						   && NumRow    == tmpEmplacement.Row
						   && NumCol    == tmpEmplacement.Column ) { trouve = 1 ; break; }
				}
			
				/*
				 * recherche si l'emplacement est occupe ou non 
				 */
				for(tmp_list_rack = list_rack;
					tmp_list_rack != NULL && trouve == 0 ; 
					tmp_list_rack = tmp_list_rack->next)
						if ( ! memcmp(&emp, tmp_list_rack->data, 3 * sizeof(gint)))
							break;
						
				if (tmp_list_rack != NULL) {		
					tmp_rack	= g_malloc(sizeof(Rack));
					tmp_vin		= g_malloc(sizeof(Vin));	
					memcpy(tmp_rack, tmp_list_rack->data, sizeof(Rack));
					memcpy(tmp_vin, tmp_rack->ptr_list_stock_data, sizeof(Vin));
				}
						
				if (tmp_list_rack == NULL || trouve == 1) 
					checkbutton[rackNum][NumRow][NumCol] = gtk_check_button_new();
				else {
					checkbutton[rackNum][NumRow][NumCol] = gtk_check_button_new ();
					GtkWidget *image;
					if (strcasecmp(tmp_vin->color, _("RED")) == 0)
						image = create_pixmap (window, "xcave_red.png");
					else
						if (strcasecmp(tmp_vin->color, _("ROSE")) == 0)
							image = create_pixmap (window, "xcave_rose.png");
						else
							image = create_pixmap (window, "xcave_white.png");
					gtk_widget_show (image);
					gtk_container_add (GTK_CONTAINER (checkbutton[rackNum][NumRow][NumCol]), image);
				}
			
				gtk_widget_set_name (checkbutton[rackNum][NumRow][NumCol], buf);
				GLADE_HOOKUP_OBJECT (window, checkbutton[rackNum][NumRow][NumCol], buf);
		
#if (defined DEBUG_LEVEL_1)
				printf("xcave_createRack:checkbutton:%s:%p:\n",buf,(void *)checkbutton[rackNum][NumRow][NumCol]);
#endif
				gtk_widget_show (checkbutton[rackNum][NumRow][NumCol]);
				gtk_table_attach (GTK_TABLE (tableRack[rackNum]), 
								  checkbutton[rackNum][NumRow][NumCol], 
								  NumCol,
								  NumCol + 1,
								  NumRow,
								  NumRow + 1,
								  (GtkAttachOptions) (GTK_FILL),
								  (GtkAttachOptions) (0), 0, 0);
				GTK_WIDGET_UNSET_FLAGS (checkbutton[rackNum][NumRow][NumCol], GTK_CAN_FOCUS);
		
				/*
				 * bouteille dans l'emplacement 
				 */
				if (tmp_list_rack != NULL) {
		                   gchar  ltooltips[128];
				   memset(ltooltips,'\0',128);
				   sprintf(ltooltips,"%s,%s,%d",tmp_vin->Appellation,tmp_vin->Nom,tmp_vin->vintage);
				   gtk_button_set_relief (GTK_BUTTON (checkbutton[rackNum][NumRow][NumCol]), GTK_RELIEF_NONE);
				   gtk_widget_set_sensitive (checkbutton[rackNum][NumRow][NumCol], TRUE);
				   if (trouve == 0) {
				      gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (checkbutton[rackNum][NumRow][NumCol]), FALSE);
				      gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips),checkbutton[rackNum][NumRow][NumCol] , ltooltips, NULL);
				   }
				   else {
				      gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (checkbutton[rackNum][NumRow][NumCol]), TRUE);
				      /*
				       * emplacement modifiable ==> deplacement du vin autorise
				       */
				      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton[rackNum][NumRow][NumCol]), TRUE);
				   }
				 
				}
				/*
				 * for window_rack need to arm a signal
				 */
				if (window == window_rack) {
					g_signal_connect ((gpointer) checkbutton[rackNum][NumRow][NumCol], "toggled",
										G_CALLBACK (on_window_rack_toggledbutton_toggled),
										NULL);
				}
		
				/*
				 * set widget name
				 */
				gchar   lab[14];
		
				sprintf(buf,"labelRackUnit_rack%d-%d-%d",rackNum,NumRow+1,NumCol+1);
				memset(lab,'\0',14);
				if ( NumRow == 0 && NumCol != 0) sprintf(lab,"%d",NumCol);
				if ( NumCol == 0 && NumRow != 0) sprintf(lab,"%d",NumRow);
			
				labelRackUnit[rackNum][NumRow][NumCol] = gtk_label_new (lab);
				gtk_widget_set_name (labelRackUnit[rackNum][NumRow][NumCol], buf);
				GLADE_HOOKUP_OBJECT (window, labelRackUnit[rackNum][NumRow][NumCol], buf); 
				gtk_widget_show (labelRackUnit[rackNum][NumRow][NumCol]);
				gtk_table_attach (GTK_TABLE (tableRack[rackNum]),
								  labelRackUnit[rackNum][NumRow][NumCol],
								  NumCol,
								  NumCol + 1,
								  NumRow,
								  NumRow + 1,
								 (GtkAttachOptions) (GTK_FILL),
								 (GtkAttachOptions) (0), 0, 0);
				gtk_misc_set_alignment (GTK_MISC (labelRackUnit[rackNum][NumRow][NumCol]), 0, 0.5);
				
				if (tmp_list_rack != NULL && window_detail_origine == BUTTON_RACK) {		
					/*
					 * keep track of the widget name in list_rack to help to 
					 * to retrieve wine data if selection is made thru 
					 * window_rack
					 */
					tmp_rack->ptr_widget_name = checkbutton[rackNum][NumRow][NumCol];
					memcpy(tmp_list_rack->data, tmp_rack, sizeof(Rack));
					g_free(tmp_vin);
				}
			}
		}
	
	}
    XCAVE_DEBUG_E;
}

GList *
xcave_rackTreatment(gchar *VinNom)
{

  GtkWidget     *checkbutton;
  gint            rackNum;
  Emplacement 	*ptemp;
  GList         *vin_list_emplacement = NULL;;

    XCAVE_DEBUG_B;

  for (rackNum=0; rackNum < rackNb; rackNum++) {
    gint    NumRow;

    // 2.2.9
    rackRowNb =(gint)g_list_nth_data(list_rackRow, rackNum);
    rackColNb = (gint)g_list_nth_data (list_rackCol, rackNum);

    // ligne
    for (NumRow=1; NumRow < rackRowNb + 1; NumRow++) {
        gint    NumCol;

		// colonne
        for (NumCol=1; NumCol < rackColNb + 1; NumCol++) {
            gchar          buf[64];

			sprintf(buf,"checkbutton%d-%d-%d",rackNum,NumRow,NumCol);
			checkbutton = (GtkWidget *) lookup_widget (window_detail, buf);

			if ((gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (checkbutton))) == 1) {
	
				ptemp = g_malloc (sizeof(Emplacement));
				memset(ptemp,'\0',sizeof(Emplacement));
				ptemp->Rack = rackNum;
				ptemp->Row    = NumRow;
				ptemp->Column = NumCol;
				strcpy(ptemp->Nom,VinNom); 
		
				vin_list_emplacement   = g_list_append(vin_list_emplacement  ,ptemp);
			}
		}
    }
  }

    XCAVE_DEBUG_E;
  return(vin_list_emplacement);
}

void
on_xcave_window_button_rack_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{

    XCAVE_DEBUG_B;
	/* 
	 * desactive window_xcave
	 */
	gtk_widget_set_sensitive(window_xcave, FALSE);

	window_rack = create_window_rack();
	window_detail_origine = BUTTON_RACK;
	xcave_createRack(window_rack, NULL);

	g_signal_connect ((gpointer) window_rack, "check_resize",
                    G_CALLBACK (on_window_rack_check_resize),
                    NULL);
	/*
	 * redimensionnement de la fenetre
	 */
	if ( strlen(WindowsRackHeight) != 0 && strlen (WindowsRackWidth) != 0 ) {
	   gtk_window_set_default_size (GTK_WINDOW (window_rack), 
									atoi(WindowsRackWidth), 
									atoi(WindowsRackHeight));
	}
	else
		gtk_widget_set_size_request (window_rack, 160, 350);


	gtk_widget_show (window_rack);

    XCAVE_DEBUG_E;
}

void
on_window_rack_destroy               (GtkObject       *object,
                                        gpointer         user_data)
{
    XCAVE_DEBUG_B;
	gtk_object_destroy (GTK_OBJECT (window_rack));

	/* 
     * reactive window_xcave
     */
	gtk_widget_set_sensitive(window_xcave, TRUE);
    XCAVE_DEBUG_B;
}

void
on_window_rack_button_close_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
    XCAVE_DEBUG_B;
	gtk_object_destroy (GTK_OBJECT (window_rack));

	/* 
     * reactive window_xcave
     */
	gtk_widget_set_sensitive(window_xcave, TRUE);
    XCAVE_DEBUG_E;
}

void
on_window_rack_toggledbutton_toggled	(GtkToggleButton	*toggledbutton,
					 gpointer         user_data)
{
    XCAVE_DEBUG_B;

	GList		*tmp_list_rack;
	ptrRack	        tmp_rack;
	// ptrVin		tmp_vin;
	
	if (gtk_toggle_button_get_active(toggledbutton) == FALSE)
		return;
	
	/* recherche si l'emplacement est occupe ou non */
	tmp_rack	= g_malloc(sizeof(Rack));
	// tmp_vin		= g_malloc(sizeof(Vin));	
	// memset(tmp_vin, '\0', sizeof(Vin));
	memset(tmp_rack, '\0', sizeof(Rack));

	for(tmp_list_rack = list_rack;
	    tmp_list_rack != NULL; 
	    tmp_list_rack = tmp_list_rack->next) {
		memcpy(tmp_rack, tmp_list_rack->data, sizeof(Rack));
		if ((void *)tmp_rack->ptr_widget_name == (void *)toggledbutton) {
		   /*
		    * 2.2.6
		    * inutile
		    * memcpy(tmp_vin, tmp_rack->ptr_list_stock_data, sizeof(Vin));
		    */
		   break;
		}
	}


	/* 
	 * desactive window_rack
	 */
	gtk_widget_set_sensitive(window_rack, FALSE);
	
	if (tmp_list_rack == NULL) {
		window_detail_origine = BUTTON_RACK_ADD;
		on_xcave_window_add_button_clicked(NULL,NULL);
		/* I could activate the button 
		   retrouver dans l'emplacement avec le triplet 
		   rack-line-colum. 
		   NOT YET IN THE POCKET */
		GtkWidget     *checkbutton;
		gint            rackNum;
		for (rackNum=0; rackNum < rackNb; rackNum++) {
			gint    NumRow;
			
			// ligne
			for (NumRow=1; NumRow < rackRowNb + 1; NumRow++) {
				gint    NumCol;
			
				// colonne
				for (NumCol=1; NumCol < rackColNb + 1; NumCol++) {
					gchar          buf[64];
			
					sprintf(buf,"checkbutton%d-%d-%d",rackNum,NumRow,NumCol);
					checkbutton = (GtkWidget *) lookup_widget (window_rack, buf);
			
					if ((gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (checkbutton))) == 1) {
						checkbutton = (GtkWidget *) lookup_widget (window_detail, buf);
						gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON (checkbutton), TRUE); /* JPH */
						break;
					}
				} /* endfor (NumCol=1; NumCol < rackColNb + 1; NumCol++) */
			} /* endfor (NumRow=1; NumRow < rackRowNb + 1; NumRow++) */
		} /* endfor (rackNum=0; rackNum < rackNb; rackNum++) */
	
	}
	else {
		window_detail_origine = BUTTON_RACK_MODIF;
		on_xcave_window_display_modif(tmp_rack->ptr_list_stock_data);
	}

	g_free(tmp_rack);

    XCAVE_DEBUG_E;
}

ptrVin 
xcave_id_selected_toggledbutton	(void)
{

	GtkWidget	*toggledbutton;

	GList		*tmp_list_rack;
	ptrRack	        tmp_rack;

	gint			rackNum;

    XCAVE_DEBUG_B;

	gint			NumRow;
	gint			NumCol;

	rackNb = g_list_length(list_rackRow);

	for (rackNum=0; rackNum < rackNb; rackNum++) {

		// 2.2.9
                rackRowNb =(gint)g_list_nth_data(list_rackRow, rackNum);
                rackColNb = (gint)g_list_nth_data (list_rackCol, rackNum);

		// ligne
		for (NumRow=1; NumRow < rackRowNb + 1; NumRow++) { 

			// colonne
			for (NumCol=1; NumCol < rackColNb + 1; NumCol++) { 
				gchar          buf[64];

				// sprintf(buf,"checkbutton_rack%d-%d-%d",rackNum,NumRow,NumCol);
				sprintf(buf,"checkbutton%d-%d-%d",rackNum,NumRow,NumCol);
				toggledbutton = (GtkWidget *) lookup_widget (window_rack, buf);
				if ((gtk_toggle_button_get_active( 
						GTK_TOGGLE_BUTTON (toggledbutton))) == TRUE) {
#if (defined DEBUG_LEVEL_1)
						printf("xcave_id_selected_toggledbutton:%d:%d:%d:\n",rackNum,NumRow,NumCol);
#endif
						gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggledbutton),FALSE);
						
						/* get associated datas */
						tmp_rack	= g_malloc(sizeof(Rack));
					
						for(tmp_list_rack = list_rack;
							tmp_list_rack != NULL; 
							tmp_list_rack = tmp_list_rack->next) {
								memcpy(tmp_rack, tmp_list_rack->data, sizeof(Rack));
								/* retrouver le bon GList */
								if ((void *)tmp_rack->ptr_widget_name == (void *)toggledbutton) {
									break;
								}
						} /* endfor(tmp_list_rack = list_rack */
						break;
				} /* endif ((gtk_toggle_button_get_active( */
			} /* endfor (NumCol=1; NumCol < rackColNb + 1; NumCol++) */
		} /* endfor (NumRow=1; NumRow < rackRowNb + 1; NumRow++)*/
	} /* endfor (rackNum=0; rackNum < rackNb; rackNum++) */

    XCAVE_DEBUG_E;
	return(tmp_rack->ptr_list_stock_data);

}
